@@ -1,6 +1,6 @@
 /*
  * WPA Supplicant / UNIX domain socket -based control interface
- * Copyright (c) 2004-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2004-2005, Jouni Malinen <j@w1.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -49,16 +49,21 @@ struct ctrl_iface_priv {
 };
 
 
+static void wpa_supplicant_ctrl_iface_send(struct ctrl_iface_priv *priv,
+					   int level, const char *buf,
+					   size_t len);
+
+
 static int wpa_supplicant_ctrl_iface_attach(struct ctrl_iface_priv *priv,
 					    struct sockaddr_un *from,
 					    socklen_t fromlen)
 {
 	struct wpa_ctrl_dst *dst;
 
-	dst = wpa_zalloc(sizeof(*dst));
+	dst = os_zalloc(sizeof(*dst));
 	if (dst == NULL)
 		return -1;
-	memcpy(&dst->addr, from, sizeof(struct sockaddr_un));
+	os_memcpy(&dst->addr, from, sizeof(struct sockaddr_un));
 	dst->addrlen = fromlen;
 	dst->debug_level = MSG_INFO;
 	dst->next = priv->ctrl_dst;
@@ -78,13 +83,13 @@ static int wpa_supplicant_ctrl_iface_detach(struct ctrl_iface_priv *priv,
 	dst = priv->ctrl_dst;
 	while (dst) {
 		if (fromlen == dst->addrlen &&
-		    memcmp(from->sun_path, dst->addr.sun_path,
-			   fromlen - sizeof(from->sun_family)) == 0) {
+		    os_memcmp(from->sun_path, dst->addr.sun_path,
+			      fromlen - sizeof(from->sun_family)) == 0) {
 			if (prev == NULL)
 				priv->ctrl_dst = dst->next;
 			else
 				prev->next = dst->next;
-			free(dst);
+			os_free(dst);
 			wpa_hexdump(MSG_DEBUG, "CTRL_IFACE monitor detached",
 				    (u8 *) from->sun_path,
 				    fromlen - sizeof(from->sun_family));
@@ -109,8 +114,8 @@ static int wpa_supplicant_ctrl_iface_level(struct ctrl_iface_priv *priv,
 	dst = priv->ctrl_dst;
 	while (dst) {
 		if (fromlen == dst->addrlen &&
-		    memcmp(from->sun_path, dst->addr.sun_path,
-			   fromlen - sizeof(from->sun_family)) == 0) {
+		    os_memcmp(from->sun_path, dst->addr.sun_path,
+			      fromlen - sizeof(from->sun_family)) == 0) {
 			wpa_hexdump(MSG_DEBUG, "CTRL_IFACE changed monitor "
 				    "level", (u8 *) from->sun_path,
 				    fromlen - sizeof(from->sun_family));
@@ -145,19 +150,19 @@ static void wpa_supplicant_ctrl_iface_receive(int sock, void *eloop_ctx,
 	}
 	buf[res] = '\0';
 
-	if (strcmp(buf, "ATTACH") == 0) {
+	if (os_strcmp(buf, "ATTACH") == 0) {
 		if (wpa_supplicant_ctrl_iface_attach(priv, &from, fromlen))
 			reply_len = 1;
 		else {
 			new_attached = 1;
 			reply_len = 2;
 		}
-	} else if (strcmp(buf, "DETACH") == 0) {
+	} else if (os_strcmp(buf, "DETACH") == 0) {
 		if (wpa_supplicant_ctrl_iface_detach(priv, &from, fromlen))
 			reply_len = 1;
 		else
 			reply_len = 2;
-	} else if (strncmp(buf, "LEVEL ", 6) == 0) {
+	} else if (os_strncmp(buf, "LEVEL ", 6) == 0) {
 		if (wpa_supplicant_ctrl_iface_level(priv, &from, fromlen,
 						    buf + 6))
 			reply_len = 1;
@@ -171,7 +176,7 @@ static void wpa_supplicant_ctrl_iface_receive(int sock, void *eloop_ctx,
 	if (reply) {
 		sendto(sock, reply, reply_len, 0, (struct sockaddr *) &from,
 		       fromlen);
-		free(reply);
+		os_free(reply);
 	} else if (reply_len == 1) {
 		sendto(sock, "FAIL\n", 5, 0, (struct sockaddr *) &from,
 		       fromlen);
@@ -194,12 +199,12 @@ static char * wpa_supplicant_ctrl_iface_path(struct wpa_supplicant *wpa_s)
 	if (wpa_s->conf->ctrl_interface == NULL)
 		return NULL;
 
-	pbuf = strdup(wpa_s->conf->ctrl_interface);
+	pbuf = os_strdup(wpa_s->conf->ctrl_interface);
 	if (pbuf == NULL)
 		return NULL;
-	if (strncmp(pbuf, "DIR=", 4) == 0) {
+	if (os_strncmp(pbuf, "DIR=", 4) == 0) {
 		dir = pbuf + 4;
-		gid_str = strstr(dir, " GROUP=");
+		gid_str = os_strstr(dir, " GROUP=");
 		if (gid_str) {
 			*gid_str = '\0';
 			gid_str += 7;
@@ -207,14 +212,14 @@ static char * wpa_supplicant_ctrl_iface_path(struct wpa_supplicant *wpa_s)
 	} else
 		dir = pbuf;
 
-	len = strlen(dir) + strlen(wpa_s->ifname) + 2;
-	buf = malloc(len);
+	len = os_strlen(dir) + os_strlen(wpa_s->ifname) + 2;
+	buf = os_malloc(len);
 	if (buf == NULL) {
-		free(pbuf);
+		os_free(pbuf);
 		return NULL;
 	}
 
-	snprintf(buf, len, "%s/%s", dir, wpa_s->ifname);
+	os_snprintf(buf, len, "%s/%s", dir, wpa_s->ifname);
 #ifdef __CYGWIN__
 	{
 		/* Windows/WinPcap uses interface names that are not suitable
@@ -227,11 +232,21 @@ static char * wpa_supplicant_ctrl_iface_path(struct wpa_supplicant *wpa_s)
 		}
 	}
 #endif /* __CYGWIN__ */
-	free(pbuf);
+	os_free(pbuf);
 	return buf;
 }
 
 
+static void wpa_supplicant_ctrl_iface_msg_cb(void *ctx, int level,
+					     const char *txt, size_t len)
+{
+	struct wpa_supplicant *wpa_s = ctx;
+	if (wpa_s == NULL || wpa_s->ctrl_iface == NULL)
+		return;
+	wpa_supplicant_ctrl_iface_send(wpa_s->ctrl_iface, level, txt, len);
+}
+
+
 struct ctrl_iface_priv *
 wpa_supplicant_ctrl_iface_init(struct wpa_supplicant *wpa_s)
 {
@@ -244,7 +259,7 @@ wpa_supplicant_ctrl_iface_init(struct wpa_supplicant *wpa_s)
 	struct group *grp;
 	char *endp;
 
-	priv = wpa_zalloc(sizeof(*priv));
+	priv = os_zalloc(sizeof(*priv));
 	if (priv == NULL)
 		return NULL;
 	priv->wpa_s = wpa_s;
@@ -253,12 +268,12 @@ wpa_supplicant_ctrl_iface_init(struct wpa_supplicant *wpa_s)
 	if (wpa_s->conf->ctrl_interface == NULL)
 		return priv;
 
-	buf = strdup(wpa_s->conf->ctrl_interface);
+	buf = os_strdup(wpa_s->conf->ctrl_interface);
 	if (buf == NULL)
 		goto fail;
-	if (strncmp(buf, "DIR=", 4) == 0) {
+	if (os_strncmp(buf, "DIR=", 4) == 0) {
 		dir = buf + 4;
-		gid_str = strstr(dir, " GROUP=");
+		gid_str = os_strstr(dir, " GROUP=");
 		if (gid_str) {
 			*gid_str = '\0';
 			gid_str += 7;
@@ -305,7 +320,8 @@ wpa_supplicant_ctrl_iface_init(struct wpa_supplicant *wpa_s)
 		goto fail;
 	}
 
-	if (strlen(dir) + 1 + strlen(wpa_s->ifname) >= sizeof(addr.sun_path))
+	if (os_strlen(dir) + 1 + os_strlen(wpa_s->ifname) >=
+	    sizeof(addr.sun_path))
 		goto fail;
 
 	priv->sock = socket(PF_UNIX, SOCK_DGRAM, 0);
@@ -314,14 +330,15 @@ wpa_supplicant_ctrl_iface_init(struct wpa_supplicant *wpa_s)
 		goto fail;
 	}
 
-	memset(&addr, 0, sizeof(addr));
+	os_memset(&addr, 0, sizeof(addr));
 	addr.sun_family = AF_UNIX;
 	fname = wpa_supplicant_ctrl_iface_path(wpa_s);
 	if (fname == NULL)
 		goto fail;
-	strncpy(addr.sun_path, fname, sizeof(addr.sun_path));
+	os_strncpy(addr.sun_path, fname, sizeof(addr.sun_path));
 	if (bind(priv->sock, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-		perror("bind(PF_UNIX)");
+		wpa_printf(MSG_DEBUG, "ctrl_iface bind(PF_UNIX) failed: %s",
+			   strerror(errno));
 		if (connect(priv->sock, (struct sockaddr *) &addr,
 			    sizeof(addr)) < 0) {
 			wpa_printf(MSG_DEBUG, "ctrl_iface exists, but does not"
@@ -346,7 +363,7 @@ wpa_supplicant_ctrl_iface_init(struct wpa_supplicant *wpa_s)
 				   "be in use - cannot override it");
 			wpa_printf(MSG_INFO, "Delete '%s' manually if it is "
 				   "not used anymore", fname);
-			free(fname);
+			os_free(fname);
 			fname = NULL;
 			goto fail;
 		}
@@ -361,23 +378,24 @@ wpa_supplicant_ctrl_iface_init(struct wpa_supplicant *wpa_s)
 		perror("chmod[ctrl_interface/ifname]");
 		goto fail;
 	}
-	free(fname);
+	os_free(fname);
 
 	eloop_register_read_sock(priv->sock, wpa_supplicant_ctrl_iface_receive,
 				 wpa_s, priv);
+	wpa_msg_register_cb(wpa_supplicant_ctrl_iface_msg_cb);
 
-	free(buf);
+	os_free(buf);
 	return priv;
 
 fail:
 	if (priv->sock >= 0)
 		close(priv->sock);
-	free(priv);
+	os_free(priv);
 	if (fname) {
 		unlink(fname);
-		free(fname);
+		os_free(fname);
 	}
-	free(buf);
+	os_free(buf);
 	return NULL;
 }
 
@@ -403,16 +421,17 @@ void wpa_supplicant_ctrl_iface_deinit(struct ctrl_iface_priv *priv)
 		close(priv->sock);
 		priv->sock = -1;
 		fname = wpa_supplicant_ctrl_iface_path(priv->wpa_s);
-		if (fname)
+		if (fname) {
 			unlink(fname);
-		free(fname);
+			os_free(fname);
+		}
 
-		buf = strdup(priv->wpa_s->conf->ctrl_interface);
+		buf = os_strdup(priv->wpa_s->conf->ctrl_interface);
 		if (buf == NULL)
 			goto free_dst;
-		if (strncmp(buf, "DIR=", 4) == 0) {
+		if (os_strncmp(buf, "DIR=", 4) == 0) {
 			dir = buf + 4;
-			gid_str = strstr(dir, " GROUP=");
+			gid_str = os_strstr(dir, " GROUP=");
 			if (gid_str) {
 				*gid_str = '\0';
 				gid_str += 7;
@@ -429,7 +448,7 @@ void wpa_supplicant_ctrl_iface_deinit(struct ctrl_iface_priv *priv)
 				perror("rmdir[ctrl_interface]");
 			}
 		}
-		free(buf);
+		os_free(buf);
 	}
 
 free_dst:
@@ -437,14 +456,24 @@ free_dst:
 	while (dst) {
 		prev = dst;
 		dst = dst->next;
-		free(prev);
+		os_free(prev);
 	}
-	free(priv);
+	os_free(priv);
 }
 
 
-void wpa_supplicant_ctrl_iface_send(struct ctrl_iface_priv *priv, int level,
-				    const char *buf, size_t len)
+/**
+ * wpa_supplicant_ctrl_iface_send - Send a control interface packet to monitors
+ * @priv: Pointer to private data from wpa_supplicant_ctrl_iface_init()
+ * @level: Priority level of the message
+ * @buf: Message data
+ * @len: Message length
+ *
+ * Send a packet to all monitor programs attached to the control interface.
+ */
+static void wpa_supplicant_ctrl_iface_send(struct ctrl_iface_priv *priv,
+					   int level, const char *buf,
+					   size_t len)
 {
 	struct wpa_ctrl_dst *dst, *next;
 	char levelstr[10];
@@ -456,12 +485,12 @@ void wpa_supplicant_ctrl_iface_send(struct ctrl_iface_priv *priv, int level,
 	if (priv->sock < 0 || dst == NULL)
 		return;
 
-	snprintf(levelstr, sizeof(levelstr), "<%d>", level);
+	os_snprintf(levelstr, sizeof(levelstr), "<%d>", level);
 	io[0].iov_base = levelstr;
-	io[0].iov_len = strlen(levelstr);
+	io[0].iov_len = os_strlen(levelstr);
 	io[1].iov_base = (char *) buf;
 	io[1].iov_len = len;
-	memset(&msg, 0, sizeof(msg));
+	os_memset(&msg, 0, sizeof(msg));
 	msg.msg_iov = io;
 	msg.msg_iovlen = 2;
 
@@ -532,7 +561,7 @@ static void wpa_supplicant_global_ctrl_iface_receive(int sock, void *eloop_ctx,
 	if (reply) {
 		sendto(sock, reply, reply_len, 0, (struct sockaddr *) &from,
 		       fromlen);
-		free(reply);
+		os_free(reply);
 	} else if (reply_len) {
 		sendto(sock, "FAIL\n", 5, 0, (struct sockaddr *) &from,
 		       fromlen);
@@ -546,7 +575,7 @@ wpa_supplicant_global_ctrl_iface_init(struct wpa_global *global)
 	struct ctrl_iface_global_priv *priv;
 	struct sockaddr_un addr;
 
-	priv = wpa_zalloc(sizeof(*priv));
+	priv = os_zalloc(sizeof(*priv));
 	if (priv == NULL)
 		return NULL;
 	priv->global = global;
@@ -564,10 +593,10 @@ wpa_supplicant_global_ctrl_iface_init(struct wpa_global *global)
 		goto fail;
 	}
 
-	memset(&addr, 0, sizeof(addr));
+	os_memset(&addr, 0, sizeof(addr));
 	addr.sun_family = AF_UNIX;
-	strncpy(addr.sun_path, global->params.ctrl_interface,
-		sizeof(addr.sun_path));
+	os_strncpy(addr.sun_path, global->params.ctrl_interface,
+		   sizeof(addr.sun_path));
 	if (bind(priv->sock, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
 		perror("bind(PF_UNIX)");
 		if (connect(priv->sock, (struct sockaddr *) &addr,
@@ -609,7 +638,7 @@ wpa_supplicant_global_ctrl_iface_init(struct wpa_global *global)
 fail:
 	if (priv->sock >= 0)
 		close(priv->sock);
-	free(priv);
+	os_free(priv);
 	return NULL;
 }
 
@@ -623,5 +652,5 @@ wpa_supplicant_global_ctrl_iface_deinit(struct ctrl_iface_global_priv *priv)
 	}
 	if (priv->global->params.ctrl_interface)
 		unlink(priv->global->params.ctrl_interface);
-	free(priv);
+	os_free(priv);
 }
