@@ -1,6 +1,6 @@
 /*
  * WPA Supplicant - testing driver interface
- * Copyright (c) 2004-2006, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2004-2006, Jouni Malinen <j@w1.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -74,14 +74,14 @@ static void wpa_driver_scan_dir(struct wpa_driver_test_data *drv,
 		return;
 
 	while ((dent = readdir(dir))) {
-		if (strncmp(dent->d_name, "AP-", 3) != 0)
+		if (os_strncmp(dent->d_name, "AP-", 3) != 0)
 			continue;
 		wpa_printf(MSG_DEBUG, "%s: SCAN %s", __func__, dent->d_name);
 
-		memset(&addr, 0, sizeof(addr));
+		os_memset(&addr, 0, sizeof(addr));
 		addr.sun_family = AF_UNIX;
-		snprintf(addr.sun_path, sizeof(addr.sun_path), "%s/%s",
-			 path, dent->d_name);
+		os_snprintf(addr.sun_path, sizeof(addr.sun_path), "%s/%s",
+			    path, dent->d_name);
 
 		if (sendto(drv->test_socket, "SCAN", 4, 0,
 			   (struct sockaddr *) &addr, sizeof(addr)) < 0) {
@@ -123,7 +123,8 @@ static int wpa_driver_test_get_scan_results(void *priv,
 	size_t num = drv->num_scanres;
 	if (num > max_size)
 		num = max_size;
-	memcpy(results, &drv->scanres, num * sizeof(struct wpa_scan_result));
+	os_memcpy(results, &drv->scanres,
+		  num * sizeof(struct wpa_scan_result));
 	return num;
 }
 
@@ -171,39 +172,46 @@ static int wpa_driver_test_associate(
 		drv->assoc_wpa_ie_len = params->wpa_ie_len;
 		if (drv->assoc_wpa_ie_len > sizeof(drv->assoc_wpa_ie))
 			drv->assoc_wpa_ie_len = sizeof(drv->assoc_wpa_ie);
-		memcpy(drv->assoc_wpa_ie, params->wpa_ie,
-		       drv->assoc_wpa_ie_len);
+		os_memcpy(drv->assoc_wpa_ie, params->wpa_ie,
+			  drv->assoc_wpa_ie_len);
 	} else
 		drv->assoc_wpa_ie_len = 0;
 
 	if (drv->test_dir && params->bssid) {
-		memset(&drv->hostapd_addr, 0, sizeof(drv->hostapd_addr));
+		os_memset(&drv->hostapd_addr, 0, sizeof(drv->hostapd_addr));
 		drv->hostapd_addr.sun_family = AF_UNIX;
-		snprintf(drv->hostapd_addr.sun_path,
-			 sizeof(drv->hostapd_addr.sun_path), "%s/AP-" MACSTR,
-			 drv->test_dir, MAC2STR(params->bssid));
+		os_snprintf(drv->hostapd_addr.sun_path,
+			    sizeof(drv->hostapd_addr.sun_path),
+			    "%s/AP-" MACSTR,
+			    drv->test_dir, MAC2STR(params->bssid));
 		drv->hostapd_addr_set = 1;
 	}
 
 	if (drv->test_socket >= 0 && drv->hostapd_addr_set) {
 		char cmd[200], *pos, *end;
+		int ret;
 		end = cmd + sizeof(cmd);
 		pos = cmd;
-		pos += snprintf(pos, end - pos, "ASSOC " MACSTR " ",
-				MAC2STR(drv->own_addr));
+		ret = os_snprintf(pos, end - pos, "ASSOC " MACSTR " ",
+				  MAC2STR(drv->own_addr));
+		if (ret >= 0 && ret < end - pos)
+			pos += ret;
 		pos += wpa_snprintf_hex(pos, end - pos, params->ssid,
 					params->ssid_len);
-		pos += snprintf(pos, end - pos, " ");
+		ret = os_snprintf(pos, end - pos, " ");
+		if (ret >= 0 && ret < end - pos)
+			pos += ret;
 		pos += wpa_snprintf_hex(pos, end - pos, params->wpa_ie,
 					params->wpa_ie_len);
-		if (sendto(drv->test_socket, cmd, strlen(cmd), 0,
+		end[-1] = '\0';
+		if (sendto(drv->test_socket, cmd, os_strlen(cmd), 0,
 			   (struct sockaddr *) &drv->hostapd_addr,
 			   sizeof(drv->hostapd_addr)) < 0) {
 			perror("sendto(test_socket)");
 			return -1;
 		}
 
-		memcpy(drv->ssid, params->ssid, params->ssid_len);
+		os_memcpy(drv->ssid, params->ssid, params->ssid_len);
 		drv->ssid_len = params->ssid_len;
 	} else
 		wpa_supplicant_event(drv->ctx, EVENT_ASSOC, NULL);
@@ -215,7 +223,7 @@ static int wpa_driver_test_associate(
 static int wpa_driver_test_get_bssid(void *priv, u8 *bssid)
 {
 	struct wpa_driver_test_data *drv = priv;
-	memcpy(bssid, drv->bssid, ETH_ALEN);
+	os_memcpy(bssid, drv->bssid, ETH_ALEN);
 	return 0;
 }
 
@@ -223,7 +231,7 @@ static int wpa_driver_test_get_bssid(void *priv, u8 *bssid)
 static int wpa_driver_test_get_ssid(void *priv, u8 *ssid)
 {
 	struct wpa_driver_test_data *drv = priv;
-	memcpy(ssid, drv->ssid, 32);
+	os_memcpy(ssid, drv->ssid, 32);
 	return drv->ssid_len;
 }
 
@@ -247,7 +255,7 @@ static int wpa_driver_test_deauthenticate(void *priv, const u8 *addr,
 	struct wpa_driver_test_data *drv = priv;
 	wpa_printf(MSG_DEBUG, "%s addr=" MACSTR " reason_code=%d",
 		   __func__, MAC2STR(addr), reason_code);
-	memset(drv->bssid, 0, ETH_ALEN);
+	os_memset(drv->bssid, 0, ETH_ALEN);
 	wpa_supplicant_event(drv->ctx, EVENT_DISASSOC, NULL);
 	return wpa_driver_test_send_disassoc(drv);
 }
@@ -259,7 +267,7 @@ static int wpa_driver_test_disassociate(void *priv, const u8 *addr,
 	struct wpa_driver_test_data *drv = priv;
 	wpa_printf(MSG_DEBUG, "%s addr=" MACSTR " reason_code=%d",
 		   __func__, MAC2STR(addr), reason_code);
-	memset(drv->bssid, 0, ETH_ALEN);
+	os_memset(drv->bssid, 0, ETH_ALEN);
 	wpa_supplicant_event(drv->ctx, EVENT_DISASSOC, NULL);
 	return wpa_driver_test_send_disassoc(drv);
 }
@@ -285,7 +293,7 @@ static void wpa_driver_test_scanresp(struct wpa_driver_test_data *drv,
 	/* SCANRESP BSSID SSID IEs */
 	res = &drv->scanres[drv->num_scanres];
 
-	memset(res, 0, sizeof(*res));
+	os_memset(res, 0, sizeof(*res));
 	if (hwaddr_aton(data, res->bssid)) {
 		wpa_printf(MSG_DEBUG, "test_driver: invalid BSSID in scanres");
 		return;
@@ -294,7 +302,7 @@ static void wpa_driver_test_scanresp(struct wpa_driver_test_data *drv,
 	pos = data + 17;
 	while (*pos == ' ')
 		pos++;
-	pos2 = strchr(pos, ' ');
+	pos2 = os_strchr(pos, ' ');
 	if (pos2 == NULL) {
 		wpa_printf(MSG_DEBUG, "test_driver: invalid SSID termination "
 			   "in scanres");
@@ -310,11 +318,9 @@ static void wpa_driver_test_scanresp(struct wpa_driver_test_data *drv,
 	res->ssid_len = len;
 
 	pos = pos2 + 1;
-	while (*pos == ' ')
-		pos++;
-	pos2 = strchr(pos, ' ');
+	pos2 = os_strchr(pos, ' ');
 	if (pos2 == NULL)
-		len = strlen(pos) / 2;
+		len = os_strlen(pos) / 2;
 	else
 		len = (pos2 - pos) / 2;
 	if (len > sizeof(ie))
@@ -331,16 +337,24 @@ static void wpa_driver_test_scanresp(struct wpa_driver_test_data *drv,
 		if (len > SSID_MAX_WPA_IE_LEN)
 			len = SSID_MAX_WPA_IE_LEN;
 		if (ipos[0] == RSN_INFO_ELEM) {
-			memcpy(res->rsn_ie, ipos, len);
+			os_memcpy(res->rsn_ie, ipos, len);
 			res->rsn_ie_len = len;
 		} else if (ipos[0] == GENERIC_INFO_ELEM) {
-			memcpy(res->wpa_ie, ipos, len);
+			os_memcpy(res->wpa_ie, ipos, len);
 			res->wpa_ie_len = len;
 		}
 
 		ipos += 2 + ipos[1];
 	}
 
+	if (pos2) {
+		pos = pos2 + 1;
+		while (*pos == ' ')
+			pos++;
+		if (os_strncmp(pos, "PRIVACY", 7) == 0)
+			res->caps |= IEEE80211_CAP_PRIVACY;
+	}
+
 	drv->num_scanres++;
 }
 
@@ -357,7 +371,7 @@ static void wpa_driver_test_assocresp(struct wpa_driver_test_data *drv,
 	}
 	if (drv->use_associnfo) {
 		union wpa_event_data event;
-		memset(&event, 0, sizeof(event));
+		os_memset(&event, 0, sizeof(event));
 		event.assoc_info.req_ies = drv->assoc_wpa_ie;
 		event.assoc_info.req_ies_len = drv->assoc_wpa_ie_len;
 		wpa_supplicant_event(drv->ctx, EVENT_ASSOCINFO, &event);
@@ -397,7 +411,7 @@ static void wpa_driver_test_mlme(struct wpa_driver_test_data *drv,
 				 const u8 *data, size_t data_len)
 {
 	struct ieee80211_rx_status rx_status;
-	memset(&rx_status, 0, sizeof(rx_status));
+	os_memset(&rx_status, 0, sizeof(rx_status));
 	ieee80211_sta_rx(drv->ctx, data, data_len, &rx_status);
 }
 
@@ -412,39 +426,39 @@ static void wpa_driver_test_receive_unix(int sock, void *eloop_ctx,
 	socklen_t fromlen = sizeof(from);
 	const size_t buflen = 2000;
 
-	buf = malloc(buflen);
+	buf = os_malloc(buflen);
 	if (buf == NULL)
 		return;
 	res = recvfrom(sock, buf, buflen - 1, 0,
 		       (struct sockaddr *) &from, &fromlen);
 	if (res < 0) {
 		perror("recvfrom(test_socket)");
-		free(buf);
+		os_free(buf);
 		return;
 	}
 	buf[res] = '\0';
 
 	wpa_printf(MSG_DEBUG, "test_driver: received %u bytes", res);
 
-	if (strncmp(buf, "SCANRESP ", 9) == 0) {
+	if (os_strncmp(buf, "SCANRESP ", 9) == 0) {
 		wpa_driver_test_scanresp(drv, &from, fromlen, buf + 9);
-	} else if (strncmp(buf, "ASSOCRESP ", 10) == 0) {
+	} else if (os_strncmp(buf, "ASSOCRESP ", 10) == 0) {
 		wpa_driver_test_assocresp(drv, &from, fromlen, buf + 10);
-	} else if (strcmp(buf, "DISASSOC") == 0) {
+	} else if (os_strcmp(buf, "DISASSOC") == 0) {
 		wpa_driver_test_disassoc(drv, &from, fromlen);
-	} else if (strcmp(buf, "DEAUTH") == 0) {
+	} else if (os_strcmp(buf, "DEAUTH") == 0) {
 		wpa_driver_test_disassoc(drv, &from, fromlen);
-	} else if (strncmp(buf, "EAPOL ", 6) == 0) {
+	} else if (os_strncmp(buf, "EAPOL ", 6) == 0) {
 		wpa_driver_test_eapol(drv, &from, fromlen,
 				      (const u8 *) buf + 6, res - 6);
-	} else if (strncmp(buf, "MLME ", 5) == 0) {
+	} else if (os_strncmp(buf, "MLME ", 5) == 0) {
 		wpa_driver_test_mlme(drv, &from, fromlen,
 				     (const u8 *) buf + 5, res - 5);
 	} else {
 		wpa_hexdump_ascii(MSG_DEBUG, "Unknown test_socket command",
 				  (u8 *) buf, res);
 	}
-	free(buf);
+	os_free(buf);
 }
 
 
@@ -452,7 +466,7 @@ static void * wpa_driver_test_init(void *ctx, const char *ifname)
 {
 	struct wpa_driver_test_data *drv;
 
-	drv = wpa_zalloc(sizeof(*drv));
+	drv = os_zalloc(sizeof(*drv));
 	if (drv == NULL)
 		return NULL;
 	drv->ctx = ctx;
@@ -465,12 +479,12 @@ static void * wpa_driver_test_init(void *ctx, const char *ifname)
 	drv->bssid[3] = 0x00;
 	drv->bssid[4] = 0x00;
 	drv->bssid[5] = 0x01;
-	memcpy(drv->ssid, "test", 5);
+	os_memcpy(drv->ssid, "test", 5);
 	drv->ssid_len = 4;
 
 	/* Generate a MAC address to help testing with multiple STAs */
 	drv->own_addr[0] = 0x02; /* locally administered */
-	sha1_prf((const u8 *) ifname, strlen(ifname),
+	sha1_prf((const u8 *) ifname, os_strlen(ifname),
 		 "wpa_supplicant test mac addr generation",
 		 NULL, 0, drv->own_addr + 1, ETH_ALEN - 1);
 
@@ -488,7 +502,7 @@ static void wpa_driver_test_close_test_socket(struct wpa_driver_test_data *drv)
 
 	if (drv->own_socket_path) {
 		unlink(drv->own_socket_path);
-		free(drv->own_socket_path);
+		os_free(drv->own_socket_path);
 		drv->own_socket_path = NULL;
 	}
 }
@@ -498,8 +512,9 @@ static void wpa_driver_test_deinit(void *priv)
 {
 	struct wpa_driver_test_data *drv = priv;
 	wpa_driver_test_close_test_socket(drv);
-	free(drv->test_dir);
-	free(drv);
+	eloop_cancel_timeout(wpa_driver_test_scan_timeout, drv, drv->ctx);
+	os_free(drv->test_dir);
+	os_free(drv);
 }
 
 
@@ -510,40 +525,40 @@ static int wpa_driver_test_attach(struct wpa_driver_test_data *drv,
 	struct sockaddr_un addr;
 	size_t len;
 
-	free(drv->own_socket_path);
+	os_free(drv->own_socket_path);
 	if (dir) {
-		len = strlen(dir) + 30;
-		drv->own_socket_path = malloc(len);
+		len = os_strlen(dir) + 30;
+		drv->own_socket_path = os_malloc(len);
 		if (drv->own_socket_path == NULL)
 			return -1;
-		snprintf(drv->own_socket_path, len, "%s/STA-" MACSTR,
-			 dir, MAC2STR(drv->own_addr));
+		os_snprintf(drv->own_socket_path, len, "%s/STA-" MACSTR,
+			    dir, MAC2STR(drv->own_addr));
 	} else {
-		drv->own_socket_path = malloc(100);
+		drv->own_socket_path = os_malloc(100);
 		if (drv->own_socket_path == NULL)
 			return -1;
-		snprintf(drv->own_socket_path, 100,
-			 "/tmp/wpa_supplicant_test-%d-%d",
-			 getpid(), counter++);
+		os_snprintf(drv->own_socket_path, 100,
+			    "/tmp/wpa_supplicant_test-%d-%d",
+			    getpid(), counter++);
 	}
 
 	drv->test_socket = socket(PF_UNIX, SOCK_DGRAM, 0);
 	if (drv->test_socket < 0) {
 		perror("socket(PF_UNIX)");
-		free(drv->own_socket_path);
+		os_free(drv->own_socket_path);
 		drv->own_socket_path = NULL;
 		return -1;
 	}
 
-	memset(&addr, 0, sizeof(addr));
+	os_memset(&addr, 0, sizeof(addr));
 	addr.sun_family = AF_UNIX;
-	strncpy(addr.sun_path, drv->own_socket_path, sizeof(addr.sun_path));
+	os_strncpy(addr.sun_path, drv->own_socket_path, sizeof(addr.sun_path));
 	if (bind(drv->test_socket, (struct sockaddr *) &addr,
 		 sizeof(addr)) < 0) {
 		perror("bind(PF_UNIX)");
 		close(drv->test_socket);
 		unlink(drv->own_socket_path);
-		free(drv->own_socket_path);
+		os_free(drv->own_socket_path);
 		drv->own_socket_path = NULL;
 		return -1;
 	}
@@ -566,43 +581,43 @@ static int wpa_driver_test_set_param(void *priv, const char *param)
 		return 0;
 
 	wpa_driver_test_close_test_socket(drv);
-	pos = strstr(param, "test_socket=");
+	pos = os_strstr(param, "test_socket=");
 	if (pos) {
 		pos += 12;
-		pos2 = strchr(pos, ' ');
+		pos2 = os_strchr(pos, ' ');
 		if (pos2)
 			len = pos2 - pos;
 		else
-			len = strlen(pos);
+			len = os_strlen(pos);
 		if (len > sizeof(drv->hostapd_addr.sun_path))
 			return -1;
-		memset(&drv->hostapd_addr, 0, sizeof(drv->hostapd_addr));
+		os_memset(&drv->hostapd_addr, 0, sizeof(drv->hostapd_addr));
 		drv->hostapd_addr.sun_family = AF_UNIX;
-		memcpy(drv->hostapd_addr.sun_path, pos, len);
+		os_memcpy(drv->hostapd_addr.sun_path, pos, len);
 		drv->hostapd_addr_set = 1;
 	}
 
-	pos = strstr(param, "test_dir=");
+	pos = os_strstr(param, "test_dir=");
 	if (pos) {
 		char *end;
-		free(drv->test_dir);
-		drv->test_dir = strdup(pos + 9);
+		os_free(drv->test_dir);
+		drv->test_dir = os_strdup(pos + 9);
 		if (drv->test_dir == NULL)
 			return -1;
-		end = strchr(drv->test_dir, ' ');
+		end = os_strchr(drv->test_dir, ' ');
 		if (end)
 			*end = '\0';
 		wpa_driver_test_attach(drv, drv->test_dir);
 	} else
 		wpa_driver_test_attach(drv, NULL);
 
-	if (strstr(param, "use_associnfo=1")) {
+	if (os_strstr(param, "use_associnfo=1")) {
 		wpa_printf(MSG_DEBUG, "test_driver: Use AssocInfo events");
 		drv->use_associnfo = 1;
 	}
 
 #ifdef CONFIG_CLIENT_MLME
-	if (strstr(param, "use_mlme=1")) {
+	if (os_strstr(param, "use_mlme=1")) {
 		wpa_printf(MSG_DEBUG, "test_driver: Use internal MLME");
 		drv->use_mlme = 1;
 	}
@@ -631,9 +646,9 @@ static int wpa_driver_test_send_eapol(void *priv, const u8 *dest, u16 proto,
 
 	wpa_hexdump(MSG_MSGDUMP, "test_send_eapol TX frame", data, data_len);
 
-	memset(&eth, 0, sizeof(eth));
-	memcpy(eth.h_dest, dest, ETH_ALEN);
-	memcpy(eth.h_source, drv->own_addr, ETH_ALEN);
+	os_memset(&eth, 0, sizeof(eth));
+	os_memcpy(eth.h_dest, dest, ETH_ALEN);
+	os_memcpy(eth.h_source, drv->own_addr, ETH_ALEN);
 	eth.h_proto = host_to_be16(proto);
 
 	io[0].iov_base = "EAPOL ";
@@ -643,22 +658,23 @@ static int wpa_driver_test_send_eapol(void *priv, const u8 *dest, u16 proto,
 	io[2].iov_base = (u8 *) data;
 	io[2].iov_len = data_len;
 
-	memset(&msg, 0, sizeof(msg));
+	os_memset(&msg, 0, sizeof(msg));
 	msg.msg_iov = io;
 	msg.msg_iovlen = 3;
-	if (memcmp(dest, drv->bssid, ETH_ALEN) == 0 || drv->test_dir == NULL) {
+	if (os_memcmp(dest, drv->bssid, ETH_ALEN) == 0 ||
+	    drv->test_dir == NULL) {
 		msg.msg_name = &drv->hostapd_addr;
 		msg.msg_namelen = sizeof(drv->hostapd_addr);
 	} else {
 		struct stat st;
-		memset(&addr, 0, sizeof(addr));
+		os_memset(&addr, 0, sizeof(addr));
 		addr.sun_family = AF_UNIX;
-		snprintf(addr.sun_path, sizeof(addr.sun_path),
-			 "%s/STA-" MACSTR, drv->test_dir, MAC2STR(dest));
+		os_snprintf(addr.sun_path, sizeof(addr.sun_path),
+			    "%s/STA-" MACSTR, drv->test_dir, MAC2STR(dest));
 		if (stat(addr.sun_path, &st) < 0) {
-			snprintf(addr.sun_path, sizeof(addr.sun_path),
-				 "%s/AP-" MACSTR,
-				 drv->test_dir, MAC2STR(dest));
+			os_snprintf(addr.sun_path, sizeof(addr.sun_path),
+				    "%s/AP-" MACSTR,
+				    drv->test_dir, MAC2STR(dest));
 		}
 		msg.msg_name = &addr;
 		msg.msg_namelen = sizeof(addr);
@@ -676,7 +692,7 @@ static int wpa_driver_test_send_eapol(void *priv, const u8 *dest, u16 proto,
 static int wpa_driver_test_get_capa(void *priv, struct wpa_driver_capa *capa)
 {
 	struct wpa_driver_test_data *drv = priv;
-	memset(capa, 0, sizeof(*capa));
+	os_memset(capa, 0, sizeof(*capa));
 	capa->key_mgmt = WPA_DRIVER_CAPA_KEY_MGMT_WPA |
 		WPA_DRIVER_CAPA_KEY_MGMT_WPA2 |
 		WPA_DRIVER_CAPA_KEY_MGMT_WPA_PSK |
@@ -720,14 +736,14 @@ wpa_driver_test_get_hw_feature_data(void *priv, u16 *num_modes, u16 *flags)
 
 	*num_modes = 1;
 	*flags = 0;
-	modes = wpa_zalloc(*num_modes * sizeof(struct wpa_hw_modes));
+	modes = os_zalloc(*num_modes * sizeof(struct wpa_hw_modes));
 	if (modes == NULL)
 		return NULL;
 	modes[0].mode = WPA_MODE_IEEE80211G;
 	modes[0].num_channels = 1;
 	modes[0].num_rates = 1;
-	modes[0].channels = wpa_zalloc(sizeof(struct wpa_channel_data));
-	modes[0].rates = wpa_zalloc(sizeof(struct wpa_rate_data));
+	modes[0].channels = os_zalloc(sizeof(struct wpa_channel_data));
+	modes[0].rates = os_zalloc(sizeof(struct wpa_rate_data));
 	if (modes[0].channels == NULL || modes[0].rates == NULL) {
 		ieee80211_sta_free_hw_features(modes, *num_modes);
 		return NULL;
@@ -773,13 +789,15 @@ static int wpa_driver_test_send_mlme(void *priv, const u8 *data,
 	io[1].iov_base = (u8 *) data;
 	io[1].iov_len = data_len;
 
-	memset(&msg, 0, sizeof(msg));
+	os_memset(&msg, 0, sizeof(msg));
 	msg.msg_iov = io;
 	msg.msg_iovlen = 2;
-	if (memcmp(dest, drv->bssid, ETH_ALEN) == 0 || drv->test_dir == NULL) {
+	if (os_memcmp(dest, drv->bssid, ETH_ALEN) == 0 ||
+	    drv->test_dir == NULL) {
 		msg.msg_name = &drv->hostapd_addr;
 		msg.msg_namelen = sizeof(drv->hostapd_addr);
-	} else if (memcmp(dest, "\xff\xff\xff\xff\xff\xff", ETH_ALEN) == 0) {
+	} else if (os_memcmp(dest, "\xff\xff\xff\xff\xff\xff", ETH_ALEN) == 0)
+	{
 		dir = opendir(drv->test_dir);
 		if (dir == NULL)
 			return -1;
@@ -793,15 +811,15 @@ static int wpa_driver_test_send_mlme(void *priv, const u8 *data,
 			    dent->d_type != DT_UNKNOWN)
 				continue;
 #endif /* _DIRENT_HAVE_D_TYPE */
-			if (strcmp(dent->d_name, ".") == 0 ||
-			    strcmp(dent->d_name, "..") == 0)
+			if (os_strcmp(dent->d_name, ".") == 0 ||
+			    os_strcmp(dent->d_name, "..") == 0)
 				continue;
 			wpa_printf(MSG_DEBUG, "%s: Send broadcast MLME to %s",
 				   __func__, dent->d_name);
-			memset(&addr, 0, sizeof(addr));
+			os_memset(&addr, 0, sizeof(addr));
 			addr.sun_family = AF_UNIX;
-			snprintf(addr.sun_path, sizeof(addr.sun_path), "%s/%s",
-				 drv->test_dir, dent->d_name);
+			os_snprintf(addr.sun_path, sizeof(addr.sun_path),
+				    "%s/%s", drv->test_dir, dent->d_name);
 
 			msg.msg_name = &addr;
 			msg.msg_namelen = sizeof(addr);
@@ -813,14 +831,14 @@ static int wpa_driver_test_send_mlme(void *priv, const u8 *data,
 		return 0;
 	} else {
 		struct stat st;
-		memset(&addr, 0, sizeof(addr));
+		os_memset(&addr, 0, sizeof(addr));
 		addr.sun_family = AF_UNIX;
-		snprintf(addr.sun_path, sizeof(addr.sun_path),
-			 "%s/AP-" MACSTR, drv->test_dir, MAC2STR(dest));
+		os_snprintf(addr.sun_path, sizeof(addr.sun_path),
+			    "%s/AP-" MACSTR, drv->test_dir, MAC2STR(dest));
 		if (stat(addr.sun_path, &st) < 0) {
-			snprintf(addr.sun_path, sizeof(addr.sun_path),
-				 "%s/STA-" MACSTR,
-				 drv->test_dir, MAC2STR(dest));
+			os_snprintf(addr.sun_path, sizeof(addr.sun_path),
+				    "%s/STA-" MACSTR,
+				    drv->test_dir, MAC2STR(dest));
 		}
 		msg.msg_name = &addr;
 		msg.msg_namelen = sizeof(addr);
